#!/usr/bin/env Rscript
args = commandArgs(trailingOnly=TRUE)

library(caTools)
library(glmnet)

library(parallel)

tab <- read.table("tab_all.txt",header=T)
ranNum <- as.numeric(args[1])

tab.cp <- tab

tab.var <- apply(tab,2,var)
n <- which(tab.var==0)

if(length(n) > 0){
tab <- tab[,-n]
}

tab <- scale(tab[,-ncol(tab)])
tab <- data.frame(tab)
tab$label <- tab.cp$label

set.seed(ranNum)
split <- sample.split(tab$label, SplitRatio = 0.70)
train <- subset(tab, split == T)
test <- subset(tab, split == F)

getTT <- function(x,data){
        d <- t.test(data[,x]~label,data)
        c(d$statistic,d$p.value)
}

no_cores <- detectCores() - 2
clust <- makeCluster(no_cores)

train.t <- parLapply(clust,1:(ncol(train)-1),getTT,train)

stopCluster(clust)

names(train.t) <- colnames(train)[1:(ncol(train)-1)]

res <- do.call(rbind,train.t)
res <- data.frame(res)
colnames(res) <- c("tstat","pval")

train.pvaln <- rownames(res)[which(abs(res$pval)<0.01)]

cname <- c()
for(i in 1:length(train.pvaln)){
        cn <- which(colnames(train)==train.pvaln[i])
        cname <- c(cname,cn)
}

train.roi <- train[,cname]
test.roi <- test[,cname]

tr <- data.frame(train.roi,label=train$label)
te <- data.frame(test.roi,label=test$label)


write.table(tr,paste("trainroi","_",ranNum,".txt",sep=""))
write.table(te,paste("testroi","_",ranNum,".txt",sep=""))


