###########################################

#Combined analysis with expert features   #

###########################################

#load library

library(MASS)



#trainset <- read.table(file="trainroi_261.txt")
#test <- read.table(file="testroi_261.txt")

args = commandArgs(trailingOnly=TRUE)

filename.train <- paste("trainroi_",args[1],".txt",sep="")
filename.test <- paste("testroi_",args[1],".txt",sep="")

trainset <- read.table(filename.train)
test <- read.table(filename.test)
beta <- 90


malignant.id <- grep("MALIGNANT", rownames(trainset))
benign.id <- grep("BENIGN", rownames(trainset))


class.label <- character(nrow(trainset))
class.label[malignant.id] <- "MALIGNANT"
class.label[benign.id] <- "BENIGN"


#identify the features generated from Krawtchouk moments and GPZM

krawtchouk.var <- grep("K", colnames(trainset))

gpzm.var <-  grep("Z", colnames(trainset))



#split data according to feature type

trainset.krawtchouk <- trainset[,krawtchouk.var]

trainset.gpzm <- trainset[,gpzm.var]



#add class label to data matrix

trainset.krawtchouk <- data.frame(class.label, trainset.krawtchouk)

trainset.gpzm <- data.frame(class.label, trainset.gpzm)

colnames(trainset.krawtchouk)[1] <- "cancer"

colnames(trainset.gpzm)[1] <- "cancer"



#dimensional reduction with Q-mode PCA

trainset.pca.krawtchouk <- prcomp(trainset.krawtchouk[,-1])



#percentage of variation explained by each PC

varexplain.krawtchouk <- (trainset.pca.krawtchouk$sdev)^2 / sum( (trainset.pca.krawtchouk$sdev)^2 ) * 100



#find the number of PC explaining up to 95% of total variation

f <- min(which(cumsum(varexplain.krawtchouk) >= beta))



trainset.pca.krawtchouk.pc <- data.frame(trainset.krawtchouk[,1], (trainset.pca.krawtchouk$x)[,1:f])

colnames(trainset.pca.krawtchouk.pc)[1] <- "cancer"



model.krawtchouk <- lda(cancer~., data=trainset.pca.krawtchouk.pc)

lda.krawtchouk <- as.matrix(trainset.pca.krawtchouk.pc[,-1]) %*% model.krawtchouk$scaling





trainset.pca.gpzm <- prcomp(trainset.gpzm[,-1])



#percentage of variation explained by each PC

varexplain.gpzm <- (trainset.pca.gpzm$sdev)^2 / sum( (trainset.pca.gpzm$sdev)^2 ) * 100



#find the number of PC explaining up to 95% of total variation

g <- min(which(cumsum(varexplain.gpzm) >= beta))



trainset.pca.gpzm.pc <- data.frame(trainset.gpzm[,1], (trainset.pca.gpzm$x)[,1:g])

colnames(trainset.pca.gpzm.pc)[1] <- "cancer"



model.gpzm <- lda(cancer~., data=trainset.pca.gpzm.pc)

lda.gpzm <- as.matrix(trainset.pca.gpzm.pc[,-1]) %*% model.gpzm$scaling



#completed processing for training set

Z <- data.frame(class.label, lda.gpzm, lda.krawtchouk)

colnames(Z) <- c("cancer", "LD.gpzm", "LD.krawtchouk")



#quick visualisation

#plot(Z$LD.gpzm, Z$LD.krawtchouk, pch=16, col=c("blue","magenta")[unclass(as.factor(Z$cancer))])



######################

#Processing test set #

######################

test.class.label <- character(nrow(test))

test.class.label[grep("MALIGNANT", rownames(test))] <- "MALIGNANT"

test.class.label[grep("BENIGN", rownames(test))] <- "BENIGN"







#split data according to feature type

test.krawtchouk <- test[,krawtchouk.var]

test.gpzm <- test[,gpzm.var]



#add class label to data matrix

test.krawtchouk <- data.frame(test.class.label, test.krawtchouk)

test.gpzm <- data.frame(test.class.label, test.gpzm)

colnames(test.krawtchouk)[1] <- "cancer"

colnames(test.gpzm)[1] <- "cancer"





G <- apply(test.krawtchouk[,-1], 1, function(k) k - apply(trainset.krawtchouk[,-1], 2, mean) )

test.pca.krawtchouk <- t(t(trainset.pca.krawtchouk$rotation) %*% G)

test.pca.krawtchouk.pc <- data.frame(test.class.label, test.pca.krawtchouk[,1:f])

colnames(test.pca.krawtchouk.pc)[1] <- "cancer"



H <- apply(test.gpzm[,-1], 1, function(k) k - apply(trainset.gpzm[,-1], 2, mean) )

test.pca.gpzm <- t(t(trainset.pca.gpzm$rotation) %*% H)

test.pca.gpzm.pc <- data.frame(test.class.label, test.pca.gpzm[,1:g])

colnames(test.pca.gpzm.pc)[1] <- "cancer"



#####################################

#Compute the LD of the test samples #

#####################################



#model.test.krawtchouk <- lda(cancer~., data=trainset.pca.krawtchouk.pc)

lda.test.krawtchouk <- as.matrix(test.pca.krawtchouk.pc[,-1]) %*% model.krawtchouk$scaling





#model.test.gpzm <- lda(cancer~., data=trainset.pca.gpzm.pc)

lda.test.gpzm <- as.matrix(test.pca.gpzm.pc[,-1]) %*% model.gpzm$scaling





#the test set result

Z.test <- data.frame(test.class.label, lda.test.gpzm, lda.test.krawtchouk)

colnames(Z.test) <- c("cancer", "LD.gpzm", "LD.krawtchouk")





######################################

#The final data set for use in GUIDE #

######################################





train <- c(rep(1, nrow(trainset)), rep(0, nrow(test)))

allsamples <- rbind(Z, Z.test)

allsamples <- data.frame(allsamples,train)



#write to a file

#this file is used as input for GUIDE to estimate the prob. malignant

write.csv(allsamples, file="cbis_guide.csv", row.names=FALSE, quote=FALSE)
write.table(allsamples,file="cbis_guide_rownames.txt")
